package com.y_ys.bench ;

import com.y_ys.bench.httpclient.AtomicINT;
import com.y_ys.bench.httpclient.AtomicLONG;
import com.y_ys.bench.httpclient.HttpCounter;
import com.y_ys.bench.httpclient.HttpHeader;
import com.y_ys.bench.httpclient.HttpThread;


/**
 * HTTPベンチマーク.
 *
 * @version 2008/11/16
 * @author masahito suzuki
 * @since MBench 1.00
 */
public class MBench implements HttpCounter {
    
    public static final String USER_AGENT = "MBench-1.02" ;
    private static final int DEFAULT_THREAD_LENGTH = 10 ;
    private static final int DEFAULT_LOOP_COUNT = 10 ;
    private static final int DEFAULT_WAIT = 0 ;
    private static final int NEXT_COUNT = 250 ;
    
    private final AtomicINT startThread = new AtomicINT( 0 ) ;
    private final AtomicINT errorCount = new AtomicINT( 0 ) ;
    private final AtomicINT successCount = new AtomicINT( 0 ) ;
    private final AtomicLONG minTime = new AtomicLONG( Long.MAX_VALUE ) ;
    private final AtomicLONG maxTime = new AtomicLONG( 0 ) ;
    
    public static final void main( String[] args ) throws Exception {
        new MBench().execution( args ) ;
    }
    
    public void startThread() {
        startThread.inc() ;
    }
    
    public int startLength() {
        return startThread.get() ;
    }
    
    public void addErrorCount() {
        errorCount.inc() ;
    }
    public int getErrorCount() {
        return errorCount.get() ;
    }
    public void addSuccessCount() {
        successCount.inc() ;
    }
    public int getSuccessCount() {
        return successCount.get() ;
    }
    
    public void minMaxTime( long time ) {
        if( minTime.get() > time ) {
            minTime.set( time ) ;
        }
        if( maxTime.get() < time ) {
            maxTime.set( time ) ;
        }
    }
    
    public long getMinTime() {
        return minTime.get() ;
    }
    
    public long getMaxTime() {
        return maxTime.get() ;
    }
    
    private String getUrl( String[] args ) {
        if( args == null || args.length <= 0 ) {
            return null ;
        }
        int len = args.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            String s = args[ i ].toLowerCase() ;
            if( s.startsWith( "-url" ) ) {
                s = s.substring( "-url".length() ) ;
                if( s.startsWith( "http://" ) || s.startsWith( "https://" ) ) {
                    return s ;
                }
            }
        }
        return null ;
    }
    
    private int getThreadLength( String[] args ) {
        if( args == null || args.length <= 0 ) {
            return DEFAULT_THREAD_LENGTH ;
        }
        int len = args.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            String s = args[ i ].toLowerCase() ;
            if( s.startsWith( "-thread" ) ) {
                String c = s.substring( "-thread".length() ) ;
                int ret = 0 ;
                try {
                    ret = Integer.parseInt( c ) ;
                } catch( Exception e ) {
                    continue ;
                }
                if( ret <= 0 ) {
                    ret = DEFAULT_THREAD_LENGTH ;
                }
                return ret ;
            }
        }
        return DEFAULT_THREAD_LENGTH ;
    }
    
    private int getLoopLength( String[] args ) {
        if( args == null || args.length <= 0 ) {
            return DEFAULT_LOOP_COUNT ;
        }
        int len = args.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            String s = args[ i ].toLowerCase() ;
            if( s.startsWith( "-loop" ) ) {
                String c = s.substring( "-loop".length() ) ;
                int ret = 0 ;
                try {
                    ret = Integer.parseInt( c ) ;
                } catch( Exception e ) {
                    continue ;
                }
                if( ret <= 0 ) {
                    ret = DEFAULT_LOOP_COUNT ;
                }
                return ret ;
            }
        }
        return DEFAULT_LOOP_COUNT ;
    }
    
    private int getWait( String[] args ) {
        if( args == null || args.length <= 0 ) {
            return DEFAULT_WAIT ;
        }
        int len = args.length ;
        for( int i = 0 ; i < len ; i ++ ) {
            String s = args[ i ].toLowerCase() ;
            if( s.startsWith( "-wait" ) ) {
                String c = s.substring( "-wait".length() ) ;
                int ret = 0 ;
                try {
                    ret = Integer.parseInt( c ) ;
                } catch( Exception e ) {
                    continue ;
                }
                if( ret <= 0 ) {
                    ret = DEFAULT_WAIT ;
                }
                return ret ;
            }
        }
        return DEFAULT_WAIT ;
    }
    
    private void execution( String[] args ) throws Exception {
        String url = getUrl( args ) ;
        if( url == null ) {
            System.out.println( "MBench -urlXXX -threadNNN -loopNNN -waitNNN" ) ;
            System.out.println( " [-urlXXX] -urlhttp://www.yahoo.co.jp" ) ;
            System.out.println( " [-threadNNN] -thread50" ) ;
            System.out.println( " [-loopNNN] -loop1000" ) ;
            System.out.println( " [-waitNNN] -wait1000" ) ;
            System.exit( 1 ) ;
        }
        int threadLen = getThreadLength( args ) ;
        int loop = getLoopLength( args ) ;
        int wait = getWait( args ) ;
        int exitCount = loop * threadLen ;
        HttpHeader header = new HttpHeader() ;
        header.add( "Connection","keep-alive" ) ;
        header.add( "Accept-Encoding","gzip" ) ;
        header.add( "Accept-Encoding","deflate" ) ;
        HttpThread[] threads = new HttpThread[ threadLen ] ;
        for( int i = 0 ; i < threadLen ; i ++ ) {
            threads[ i ] = new HttpThread( this,url,header,wait ) ;
        }
        while( true ) {
            if( startLength() >= threadLen ) {
                break ;
            }
        }
        for( int i = 0 ; i < threadLen ; i ++ ) {
            threads[ i ].startThread() ;
        }
        System.out.println( "start[url:" + url + " thread:" + threadLen + " loop:" + loop + " wait:" + wait + "]" ) ;
        long time = System.currentTimeMillis() ;
        int nextCount = NEXT_COUNT ;
        int successCount ;
        int cnt = 0 ;
        long t ;
        while( true ) {
            successCount = getSuccessCount() ;
            if( nextCount <= successCount ) {
                System.out.print( "#" ) ;
                nextCount = successCount + NEXT_COUNT ;
                if( successCount >= exitCount ) {
                    time = System.currentTimeMillis() - time ;
                    break ;
                }
                else if( cnt >= 50 ) {
                    t = System.currentTimeMillis() - time ;
                    System.out.println( "\n(throughput)1sec=" + ( successCount / ( int )(t/1000L) )  + "access\n" +
                        "(count:" + successCount + " time:" + ( System.currentTimeMillis() - time ) + "msec)\n" +
                        "min:" + getMinTime() + " max:" + getMaxTime() ) ;
                    cnt = 0 ;
                }
                else {
                    cnt ++ ;
                }
            }
            Thread.sleep( 5L ) ;
        }
        System.out.println( "exit("+exitCount+")-(error:" + getErrorCount() + ")-time:" + time + "msec" ) ;
        System.out.println( "(throughput)1sec=" + ( successCount / (( int )time/1000L) )  + "access" ) ;
        System.out.println( "min:" + getMinTime() + " max:" + getMaxTime() ) ;
        System.exit( 0 ) ;
    }
    
}

