package com.y_ys.bench.httpclient ;

/**
 * HTTP結果情報.
 *
 * @version 2008/11/16
 * @author masahito suzuki
 * @since MBench 1.00
 */
public class HttpResult {
    private int state = -1 ;
    private String contentType = null ;
    private String connection = null ;
    private String keepAliveTime = null ;
    private int keepAliveCount = 0 ;
    private boolean isKeepAlive = false ;
    private byte[] body = null ;
    
    public HttpResult() {
    }
    
    public void clear() {
        state = -1 ;
        contentType = null ;
        connection = null ;
        keepAliveTime = null ;
        keepAliveCount = 0 ;
        isKeepAlive = false ;
        body = null ;
    }
    
    public void setState( int state ) {
        this.state = state ;
    }
    
    public int getState() {
        return state ;
    }
    
    public void setContentType( String contentType ) {
        this.contentType = contentType ;
    }
    
    public String getContentType() {
        return contentType ;
    }
    
    public void setConnection( String connection ) {
        if( connection == null ) {
            this.connection = null ;
            this.isKeepAlive = false ;
        }
        else {
            connection = connection.toLowerCase() ;
            if( connection.equals( "keep-alive" ) ) {
                this.isKeepAlive = true ;
            }
            else {
                this.isKeepAlive = false ;
            }
            this.connection = connection ;
        }
    }
    
    public String getConnection() {
        return connection ;
    }
    
    public void setKeepAlive( String keepAlive ) {
        if( keepAlive != null ) {
            keepAlive = keepAlive.toLowerCase() ;
            String[] x = null ;
            int p = keepAlive.indexOf( "," ) ;
            if( p != -1 ) {
                x = new String[ 2 ] ;
                x[0] = keepAlive.substring( 0,p ).trim() ;
                x[1] = keepAlive.substring( p+1 ).trim() ;
            }
            else {
                x = new String[ 1 ] ;
                x[0] = keepAlive.trim() ;
            }
            int len = x.length ;
            for( int i = 0 ; i < len ; i ++ ) {
                String s = x[ i ] ;
                if( s.startsWith( "timeout=" ) ) {
                    keepAliveTime = s.substring( "timeout=".length() ) ;
                }
                else if( s.startsWith( "max=" ) ) {
                    keepAliveCount = Integer.parseInt( s.substring( "max=".length() ) ) ;
                }
            }
        }
        else {
            keepAliveTime = null ;
            keepAliveCount = -1 ;
        }
    }
    
    public String getKeepAliveTime() {
        return keepAliveTime ;
    }
    
    public int getKeepAliveCount() {
        return keepAliveCount ;
    }
    
    public void setBody( byte[] body ) {
        this.body = body ;
    }
    
    public byte[] getBody() {
        return body ;
    }
    
    public boolean isKeepAlive() {
        return isKeepAlive ;
    }
    
    public String toString() {
        StringBuilder buf = new StringBuilder() ;
        buf.append( "state:" ).
            append( state ).
            append( " contentType:" ).
            append( contentType ).
            append( " connection:" ).
            append( connection ).
            append( " isKeepAlive:" ).
            append( isKeepAlive ).
            append( " keepAlive:[time:" ).
            append( keepAliveTime ).
            append( " count:" ).
            append( keepAliveCount ).
            append( "]" ).
            append( " bodyLen:" ).
            append( ( ( body == null ) ? 0 : body.length ) ) ;
        return buf.toString() ;
    }
}

